#: Object types for object ids
object_type = {}
object_type["dynamic_global_property"] = 0
object_type["reserved0"] = 1
object_type["account"] = 2
object_type["witness"] = 3
object_type["transaction"] = 4
object_type["block_summary"] = 5
object_type["chain_property"] = 6
object_type["witness_schedule"] = 7
object_type["comment"] = 8
object_type["category"] = 9
object_type["comment_vote"] = 10
object_type["vote"] = 11
object_type["witness_vote"] = 12
object_type["limit_order"] = 13
object_type["feed_history"] = 14
object_type["convert_request"] = 15
object_type["liquidity_reward_balance"] = 16
object_type["operation"] = 17
object_type["account_history"] = 18
