#: Object types for object ids
from typing import Dict

object_type: Dict[str, int] = {}
object_type["null"] = 0
object_type["base"] = 1
object_type["account"] = 2
object_type["OBJECT_TYPE_COUNT"] = 3
